@@ -55,6 +55,7 @@ extern int optind;
 #include "problem.h"
 #include "jfs_user.h"
 #include "../version.h"
+#include <ext2fs/nls.h>
 
 /* Command line options */
 static int cflag;		/* check disk */
@@ -1784,6 +1785,15 @@ print_unsupp_features:
 		goto get_newer;
 	}
 
+	if (ext2fs_has_feature_fname_encoding(sb)) {
+		fs->encoding = nls_load_table(sb->s_encoding);
+		if (!fs->encoding) {
+			log_err(ctx, _("%s has unsupported encoding: %0x\n"),
+				ctx->filesystem_name, sb->s_encoding);
+			goto get_newer;
+		}
+	}
+
 	/*
 	 * If the user specified a specific superblock, presumably the
 	 * master superblock has been trashed.  So we mark the
